{
Copyright 2000 (c) Aurlien GATEAU aka Bug*Killer
aurelien.gateau@mail.dotcom.fr

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit CutBox;

interface
uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls, RectBox,CutFile, CutIt_gui,DocView;

type
	TCutBox=class(TRectBox,IView)
    private
		FCut:TCut;
		FFormCutIt:TFormCutIt;

		procedure SetBoundsFromCut;
		function GetScrollBoxOffset: TPoint;

	public
		constructor CreateFromCut(AOwner:TComponent;Cut:TCut;FormCutIt:TFormCutIt);
		procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure OnRectChanged; override;

		procedure OnDocMessage(Msg:TDVMsg);

		property Cut:TCut read FCut;
    end;

implementation


constructor TCutBox.CreateFromCut(AOwner:TComponent;Cut:TCut;FormCutIt:TFormCutIt);
begin
	Create(AOwner);

	FCut:=Cut;
	FFormCutIt:=FormCutIt;

// Init TRectBox fields
	FrameType:=ftInner;
	HidePassiveFrame:=false;
	BoxSize:=4;

// Init BoundsRect
	SetBoundsFromCut;
end;


{

TCutBox.MouseDown
Notify the form that a cut is selected

}
procedure TCutBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	inherited MouseDown(Button,Shift,X,Y);
	FormCutIt.CutSelected:=true;
end;


{

TCutBox.OnDocMessage
We receive a message when the cut has changed,
we must update BoundsRect

}
procedure TCutBox.OnDocMessage(Msg:TDVMsg);
begin
	if Msg=MSG_CUT_RECTCHANGED then begin
		SetBoundsFromCut;
		parent.invalidate;
	end;
end;


{

TCutBox.OnRectChanged
Apply changes to the TCut object

}
procedure TCutBox.OnRectChanged();
var
	ScrollOffset:TPoint;
	Rect:TRect;
begin
	ScrollOffset:=GetScrollBoxOffset;
	Rect:=InnerRect;
		
	Rect.left:=Rect.left+ScrollOffset.x;
	Rect.top:=Rect.top+ScrollOffset.y;
	Rect.right:=Rect.right+ScrollOffset.x;
	Rect.bottom:=Rect.bottom+ScrollOffset.y;
	
	FCut.Rect:=Rect;
end;


{

TCutBox.SetBoundsFromCut
Updates BoundsRect according to FCut.Rect

}
procedure TCutBox.SetBoundsFromCut;
var
	BoxOffset:Integer;
	ScrollOffset:TPoint;
	Rect:TRect;
begin
	Rect:=FCut.Rect;
	
	BoxOffset:=GetBoxOffset;
	ScrollOffset:=GetScrollBoxOffset;
	
	Rect.Left:=Rect.Left-BoxOffset-ScrollOffset.x;
	Rect.Top:=Rect.Top-BoxOffset-ScrollOffset.y;
	Rect.Right:=Rect.Right+BoxOffset-ScrollOffset.x;
	Rect.Bottom:=Rect.Bottom+BoxOffset-ScrollOffset.y;
	BoundsRect:=Rect;
end;


{

TCutBox.GetScrollBoxOffset
Calculate the scroll box offset (When we don't see the image top left corner)

}
function TCutBox.GetScrollBoxOffset: TPoint;
begin
	Result.x:=FFormCutIt.ScrollBox.HorzScrollBar.ScrollPos;
	Result.y:=FFormCutIt.ScrollBox.VertScrollBar.ScrollPos;
end;

end.
