{
Copyright 2000 (c) Aurlien GATEAU aka Bug*Killer
aurelien.gateau@mail.dotcom.fr

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit CutFile;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, IniFiles,Vector,Log,DocView;

const
	INI_MAIN = 'Main';
	INI_MAIN_FILENAME = 'Filename';
	INI_MAIN_OUTPUTFOLDER = 'OutputFolder';
	INI_CUT = 'Cut';
	INI_CUT_FILENAME = 'Filename';
	INI_CUT_POSX = 'PosX';
	INI_CUT_POSY = 'PosY';
	INI_CUT_SIZEX = 'SizeX';
	INI_CUT_SIZEY = 'SizeY';

	MSG_CUTFILE_MODIFIED=0;

	MSG_CUT_RECTCHANGED=0;
	MSG_CUT_FILENAMECHANGED=1;

type
	TCutFile=class;


	TCut=class(TObject)
	private
		FParent:TCutFile;
		FFileName:string;
		FCutRect:TRect;
		FDoc:TDoc;
		procedure SetCutRect(Rect:TRect);
		procedure SetFileName(FileName:string);
	public
		constructor Create(Parent:TCutFile;Section:string); overload;
		constructor Create(Parent:TCutFile;Rect:TRect); overload;
		destructor Destroy; override;

		procedure DoCut;
		procedure Save(Section:string);

		property Doc: TDoc read FDoc;
		property Rect: TRect read FCutRect write SetCutRect;
		property FileName:string read FFileName write SetFileName;
	end;

	TCutFile=class(TVector)
	private
		FBmpFileName:string;
		FIniFileName:string;
		FOutputFolder:string;
		FBmp:TBitmap;
		FDoc:TDoc;

		function GetCanvas: TCanvas;
		procedure SetBmpFileName(Value:string);
		procedure SetOutputFolder(Value:string);
	public

		constructor Create;
		destructor Destroy; override;

		procedure Load(IniFileName:string);
		procedure Save;
		procedure SaveAs(IniFileName:string);
		procedure DoCut;

		function AddCut: TCut;
		procedure RemoveCut(Cut:TCut);

		property Count read FCount;
		property Canvas: TCanvas read GetCanvas;
		property Doc: TDoc read FDoc;
		property Bitmap: TBitmap read FBmp;
		property OutputFolder: string read FOutputFolder write SetOutputFolder;
		property BmpFileName: string read FBmpFileName write SetBmpFileName;
		property IniFileName: string read FIniFileName;
	end;


implementation

{-TCut------------------------------------------------------------}
constructor TCut.Create(Parent:TCutFile;Section:string);
var
	IniFile:TIniFile;
begin
	inherited Create;
	FDoc:=TDoc.Create;

	FParent:=Parent;

    IniFile:=TIniFile.Create(FParent.IniFileName);
	FFileName:=IniFile.ReadString(Section,INI_CUT_FILENAME,'');

	FCutRect:=bounds(   IniFile.ReadInteger(Section,INI_CUT_POSX,0),
                        IniFile.ReadInteger(Section,INI_CUT_POSY,0),
						IniFile.ReadInteger(Section,INI_CUT_SIZEX,0),
						IniFile.ReadInteger(Section,INI_CUT_SIZEY,0));

    IniFile.Free;
end;


constructor TCut.Create(Parent:TCutFile;Rect:TRect);
begin
	inherited Create;
	FDoc:=TDoc.Create;

	FParent:=Parent;
	FCutRect:=Rect;
end;


destructor TCut.Destroy;
begin
	FDoc.Destroy;
	inherited destroy;
end;


procedure TCut.Save(Section:string);
var
    IniFile:TIniFile;
begin
	IniFile:=TIniFile.Create(FParent.IniFileName);
	try
        IniFile.WriteString(Section,INI_CUT_FILENAME,FFileName);
        IniFile.WriteInteger(Section,INI_CUT_POSX,FCutRect.Left);
        IniFile.WriteInteger(Section,INI_CUT_POSY,FCutRect.Top);
        IniFile.WriteInteger(Section,INI_CUT_SIZEX,FCutRect.Right-FCutRect.Left);
        IniFile.WriteInteger(Section,INI_CUT_SIZEY,FCutRect.Bottom-FCutRect.Top);
	finally
		IniFile.Free;
	end;
end;


procedure TCut.DoCut;
var
    Bmp:TBitmap;
    DestRect:TRect;
begin
    DestRect.Left:=0;
    DestRect.Top:=0;
    DestRect.Right:=FCutRect.Right-FCutRect.Left;
    DestRect.Bottom:=FCutRect.Bottom-FCutRect.Top;

    Bmp:=TBitmap.Create;
    try
        Bmp.Width:=DestRect.Right;
        Bmp.Height:=DestRect.Bottom;
        Bmp.Canvas.CopyRect(DestRect,FParent.Canvas,FCutRect);
        Bmp.SaveToFile(FParent.OutputFolder + FFileName);
    finally
        Bmp.Free;
    end;
end;


procedure TCut.SetCutRect(Rect:TRect);
begin
	FCutRect:=Rect;
	FParent.Doc.SendMessage(MSG_CUTFILE_MODIFIED);
	FDoc.SendMessage(MSG_CUT_RECTCHANGED);
end;

procedure TCut.SetFileName(FileName:string);
begin
	FFileName:=FileName;
	FParent.Doc.SendMessage(MSG_CUTFILE_MODIFIED);
	FDoc.SendMessage(MSG_CUT_FILENAMECHANGED);
end;

{-TCutFile--------------------------------------------------------}
constructor TCutFile.Create;
begin
	inherited Create;
	FDoc:=TDoc.Create;
end;


destructor TCutFile.Destroy;
begin
	FDoc.Free;
	inherited Destroy;
end;


function TCutFile.AddCut: TCut;
var
	Cut:TCut;
begin
	Cut:=TCut.Create(self,Rect(0,0,12,12));
	Add(Cut);
	Result:=Cut;
end;


procedure TCutFile.RemoveCut(Cut:TCut);
var
	Num:integer;
begin
	Num:=IndexOf(Cut);
	if Num=-1 then exit;
	Delete(Num);
end;


function TCutFile.GetCanvas(): TCanvas;
begin
    Result:=FBmp.Canvas;
end;


procedure TCutFile.Load(IniFileName:string);
var
	IniFile:TIniFile;
	NbSections:integer;
	Num:integer;
    Cut:TCut;
begin
	FIniFileName:=IniFileName;
	IniFile:=TIniFile.Create(FIniFileName);
	try
	{Main Bmp}
		FBmpFileName:=IniFile.ReadString(INI_MAIN,INI_MAIN_FILENAME,'');
		if FBmpFileName='' then exit;

		FBmp.Free;
		FBmp:=TBitmap.Create;
		FBmp.LoadFromFile(FBmpFileName);

	{Output folder}
		SetOutputFolder(IniFile.ReadString(INI_MAIN,INI_MAIN_OUTPUTFOLDER,''));

	{Resize the cut list}
		NbSections:=0;
		while IniFile.SectionExists(INI_CUT + IntToStr(NbSections)) do NbSections:=NbSections+1;
		SetCapacity(NbSections);

	{Cuts}
		for Num:=0 to NbSections-1 do
		begin
            Cut:=TCut.Create(Self,INI_CUT+IntToStr(Num));
            Add( Cut );
		end;
	finally
		IniFile.Free;
	end;
end;


procedure TCutFile.SaveAs(IniFileName:string);
begin
	FIniFileName:=IniFileName;
	Save;
end;


procedure TCutFile.Save;
var
	IniFile:TIniFile;
    Num:Integer;
begin
// Delete the file if it exists (If we don't do this, deleted section will remain there)
	if FileExists(FIniFileName) then DeleteFile(FIniFileName);

// Create the file
	IniFile:=TIniFile.Create(FIniFileName);
	try
    // Save Main settings
        IniFile.WriteString(INI_MAIN,INI_MAIN_FILENAME,FBmpFileName);
        IniFile.WriteString(INI_MAIN,INI_MAIN_OUTPUTFOLDER,FOutputFolder);

    // Save cuts
		for Num:=0 to Count-1 do
		begin
			TCut(Items[Num]).Save(INI_CUT+IntToStr(Num));
		end;
	finally
		IniFile.Free;
	end;
end;


procedure TCutFile.DoCut;
var
    Num:integer;
begin
    for Num:=0 to FCount-1 do
    begin
        TCut(FItems[Num]).DoCut;
    end;
end;


procedure TCutFile.SetBmpFileName(Value:string);
begin
	FBmpFileName:=Value;
	FBmp.Free;
	FBmp:=TBitmap.Create;
	FBmp.LoadFromFile(FBmpFileName);
end;


procedure TCutFile.SetOutputFolder(Value:string);
begin
	FOutputFolder:=Value;
	if FOutputFolder<>'' then begin
		if FOutputFolder[Length(FOutputFolder)]<>'\' then begin
			FOutputFolder:=FOutputFolder + '\';
		end
	end;

end;

end.
