unit RectBox;
{
	As a thank you for all those magnificent Freeware components I learned from.

	TRectBox is mainly a Canvas that can be dragged and resized during runtime.
	The appearance of the Canvas can be modified using the OnPaint event.
	Use TRectBox whenever you need to let the user define a rectangular area at
	runtime.

	TRectBox is pretty much a standard Graphic control with the exception of the
	simulated focusing mechanism. Focus shift will work automatically between
	TRectBoxes leaving the windows focus were it is. Unfocusing must be done in
	code with the	RemoveAllFocus procedure.

	Paul van Dinther  dinther@geocities.com 02/01/2000

	Bug*Killer note : I heavily modified the original TRectBox component
}

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls, CommCtrl,Log;

type
	TFrameType = (ftOuter,ftInner,ftMiddle);
	TDragBox = (dbNone,dbDrag,dbTopLeft,dbTopMid,dbTopRight,dbMidLeft,dbMidRight,dbBottomLeft,dbBottomMid,dbBottomRight);

	TRectBox = class(TGraphicControl)
	private
		{ Private declarations }
		FAutoForeGround: Boolean;
		FFrameType: TFrameType;
		FHideFrame: Boolean;
		FFrameColor: TColor;
		FBoxSize: Integer;
		FBoxColor: TColor;
		FActiveFrameColor: TColor;
		FPassiveFrameColor: TColor;
		FDrawFrame: Boolean;
		FDrawBoxes: Boolean;
		FDragBox: TDragBox;
		FPosX: Integer;
		FPosY: Integer;

		procedure SetHideFrame(Value: Boolean);
        procedure SetFrameRect(PLeft,PTop,PWidth,PHeight: Integer;DragBox:TDragBox);
		procedure SetFrameType(Value: TFrameType);
		procedure SetBoxSize(Value: Integer);
		procedure SetBoxColor(Value: TColor);
		procedure SetActiveFrameColor(Value: TColor);
		procedure SetPassiveFrameColor(Value: TColor);
		procedure SetDrawFrame(Value: Boolean);
		procedure SetDrawBoxes(Value: Boolean);
		procedure SetInnerRect(Value: TRect);
		function GetInnerRect: TRect;

	protected
		procedure paint; override;
		procedure OnRectChanged; virtual;
		function GetBoxOffset: integer;
	public
		constructor Create(AOwner: TComponent); override;
		destructor destroy; override;

		procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure MouseMove(Shift: TShiftState; X,Y: Integer); override;
	published
		property AutoForeGround: Boolean read FAutoForeGround write FAutoForeGround;
		property FrameType: TFrameType read FFrameType write SetFrameType;
		property HidePassiveFrame: Boolean read FHideFrame write SetHideFrame;
		property BoxSize: Integer read FBoxSize write SetBoxSize;
		property BoxColor: TColor read FBoxColor write SetBoxColor;
		property ActiveFrameColor: TColor read FActiveFrameColor write SetActiveFrameColor;
		property PassiveFrameColor: TColor read FPassiveFrameColor write SetPassiveFrameColor;
		property DrawFrame: Boolean read FDrawFrame write SetDrawFrame;
		property DrawBoxes: Boolean read FDrawBoxes write SetDrawBoxes;
		property InnerRect: TRect read GetInnerRect write SetInnerRect;

		class function GetLastFocused: TRectBox;
	end;

implementation

var
	GLastFocused:TRectBox;

function between(PVal1,PVal2,PVal3: Integer):Boolean;
begin
	if (PVal1 > PVal2) and (PVal1 < PVal3) then Result := True
	else if (PVal1 > PVal3) and (PVal1 < PVal2) then result := True
	else Result := False;
end;


class function TRectBox.GetLastFocused: TRectBox;
begin
	Result:=GLastFocused;
end;


{

TRectBox constructor

}
constructor TRectBox.Create(AOwner: TComponent);
begin
	inherited create(AOwner);

	FBoxColor := clBlack;
	FActiveFrameColor := clBlack;
	FPassiveFrameColor := clGray;
	FBoxSize := 5;
	FDrawBoxes := True;
	FDrawFrame := True;
	FFrameType := ftOuter;
	FAutoForeGround := True;
	ControlStyle:=ControlStyle + [csCaptureMouse];
end;


{

TRectBox destructor

}
destructor TRectBox.destroy;
begin
	if GLastFocused=self then GLastFocused:=nil;
	inherited;
end;

{

TRectBox.Paint

}
procedure TRectBox.Paint;
var
	FrameRect: TRect;
	BoxOffset:integer;
begin
// Calculate FrameRect
	BoxOffset:=GetBoxOffset;
	FrameRect:=rect(BoxOffset,BoxOffset,Width-BoxOffset,Height-BoxOffset);


// Draw the frame
	if DrawFrame or not FHideFrame then begin
		if FDrawFrame then begin
			Canvas.pen.color := FFrameColor;
			Canvas.MoveTo(FrameRect.Left,FrameRect.Top);
			Canvas.LineTo(FrameRect.Right-1,FrameRect.Top);
			Canvas.LineTo(FrameRect.Right-1,FrameRect.Bottom-1);
			Canvas.LineTo(FrameRect.Left,FrameRect.Bottom-1);
			Canvas.LineTo(FrameRect.Left,FrameRect.Top);
		end;
	end;
	//Draw the drag boxes
	if FDrawBoxes and ((GLastFocused=self) or (csDesigning in ComponentState)) then begin		Canvas.Brush.Color:=FBoxColor;
		Canvas.FillRect(rect(0,0,FBoxSize,FBoxSize));
		Canvas.FillRect(rect( (Width-FBoxSize) div 2,0, (Width+FBoxSize) div 2 ,FBoxSize));
		Canvas.FillRect(rect(Width - FBoxSize,0,Width,FBoxSize));
		Canvas.FillRect(rect(Width - FBoxSize,(Height - FBoxSize) div 2, Width, (Height + FBoxSize) div 2));
		Canvas.FillRect(rect(Width - FBoxSize,Height - FBoxSize,Width,Height));
		Canvas.FillRect(rect((Width-FBoxSize) div 2,Height - FBoxSize,(Width+FBoxSize) div 2,Height));
		Canvas.FillRect(rect(0,Height-FBoxSize,FBoxSize,Height));
		Canvas.FillRect(rect(0,(Height-FBoxSize) div 2,FBoxSize,(Height+FBoxSize) div 2));
	end;
end;


{

TRectBox.MouseDown

}
procedure TRectBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	OldFocused:TRectBox;
begin
	FPosX := X;
	FposY := Y;
	if FAutoForeGround then	BringToFront;
	FFrameColor := FActiveFrameColor;

// Set "focus"
	OldFocused:=GLastFocused;
	GLastFocused:=self;
	if OldFocused<>nil then
		OldFocused.invalidate;

//	if FDragBox=dbNone then FDragBox := dbDrag;
	invalidate;
end;


{

TRectBox.MouseMove

}
procedure TRectBox.MouseMove(Shift: TShiftState; X,Y: Integer);
var
	MidHeightStart,MidHeightEnd: Integer;
	MidWidthStart,MidWidthEnd: Integer;
begin
	if GLastFocused<>self then begin
		Cursor:=crDefault;
		FDragBox:=dbNone;
		exit;
	end;

//Prepare some variables
	MidHeightStart := (Height - FBoxSize) div 2;
	MidHeightEnd := MidHeightStart + FBoxSize;
	MidWidthStart := (Width - FBoxSize) div 2;
	MidWidthEnd := MidWidthStart + FBoxSize;


// Update cursor and FDragBox if not clicked
	if not (ssLeft in Shift) then begin
	// Default cursor state
		Cursor:=crDefault;
		FDragBox:=dbNone;
	// Left boxes
		if between(X,0,FBoxSize) then begin
			if between(Y,0,FBoxSize) then begin
				cursor := crSizeNWSE;
				FDragBox := dbTopLeft;
			end else if between(Y,MidHeightStart,MidHeightEnd) then begin
				cursor := crSizeWE;
				FDragBox := dbMidLeft;
			end else if between(Y,Height - FBoxSize,Height) then begin
				cursor := crSizeNESW;
				FDragBox := dbBottomLeft;
			end
		end
		else
		// Right boxes
			if between(X,Width - FBoxSize,Width) then begin
				if between(Y,0,FBoxSize) then begin
					cursor := crSizeNESW;
					FDragBox := dbTopRight;
				end else if between(Y,MidHeightStart,MidHeightEnd) then begin
					Cursor := crSizeWE;
					FDragBox := dbMidRight;
				end else if between(Y,Height - FBoxSize,Height) then begin
					cursor := crSizeNWSE;
					FDragBox := dbBottomRight;
				end
			end
			else
			// Middle boxes
				if between(X,MidWidthStart,MidWidthEnd) then begin
					if between(Y,0,FBoxSize) then begin
						cursor := crSizeNS;
						FDragBox := dbTopMid;
					end else if between(Y,Height - FBoxSize,Height) then begin
						cursor := crSizeNS;
						FDragBox := dbBottomMid;
					end
				end
				else
				// Inside
					if between(X,FBoxSize,Width-FBoxSize) and between(Y,FBoxSize,Height-FBoxSize) then begin
						Cursor:=crSize;
						FDragBox:=dbDrag;
					end

	end	else begin
	// Clicked : Make size changes
		case FDragBox of
			dbTopLeft:
				SetFrameRect(Left + X,Top + Y,Width - X,Height - Y,FDragBox);
			dbTopRight:
				SetFrameRect(Left,Top + Y,X,Height - Y,FDragBox);
			dbTopMid:
				SetFrameRect(Left,Top + Y,Width,Height - Y,FDragBox);
			dbMidLeft:
				SetFrameRect(Left + X,Top,Width -X,Height,FDragBox);
			dbBottomright:
				SetFrameRect(Left,Top,X,Y,FDragBox);
			dbBottomLeft:
				SetFrameRect(Left + X,Top,Width -X,Y,FDragBox);
			dbBottomMid:
				SetFrameRect(Left,Top,Width,Y,FDragBox);
			dbMidRight:
				SetFrameRect(Left,Top,X,Height,FDragBox);
			dbDrag:
				SetFrameRect(Left + X - FPosX,Top + Y - FPosY,Width,Height,FDragBox);
		end;
	end;
end;


{

TRectBox.SetFrameRect
Defines the bound rect and calls TRectBox.OnRectChanged

}
procedure TRectBox.SetFrameRect(PLeft,PTop,PWidth,PHeight: Integer;DragBox:TDragBox);
begin
// Horizontal adjust
	if (PWidth < (3 * FBoxSize)) then begin
		if (DragBox=dbTopLeft) or (DragBox=dbMidLeft) or (DragBox=dbBottomLeft) then
			PLeft:=PLeft+PWidth-3 * FBoxSize;
		PWidth:=3 * FBoxSize;
	end;

// Vertical adjust
	if (PHeight < (3 * FBoxSize)) then begin
		if (DragBox=dbTopLeft) or (DragBox=dbTopMid) or (DragBox=dbTopRight) then
			PTop:=PTop+PHeight-3 * FBoxSize;
		PHeight:=3 * FBoxSize;
	end;

// If no changes were made then quit
	if (PLeft=Left) and (PTop=Top) and (PWidth=Width) and (PHeight=Height) then exit;

// Update
	SetBounds(PLeft,PTop,PWidth,PHeight);
	invalidate;
	OnRectChanged;
end;


{

TRectBox.OnRectChanged
Function which can be overrided to start some code when the user changes the rect box

}
procedure TRectBox.OnRectChanged;
begin
end;


{

GetBoxOffset
Calculates the box offset according to FFrameType

}
function TRectBox.GetBoxOffset: Integer;
begin
	case FFrameType of
		ftOuter: Result := 0;
		ftInner: Result := FBoxSize;
		ftMiddle: Result := (FBoxSize div 2);
	else
		Result:=0;
	end;
end;


{-Property functions-------------------------------------------}
function TRectBox.GetInnerRect: TRect;
var
	BoxOffset:Integer;
begin
	BoxOffset:=GetBoxOffset;

	Result:=BoundsRect;
	Result.Left:=Result.Left+BoxOffset;
	Result.Top:=Result.Top+BoxOffset;
	Result.Right:=Result.Right-BoxOffset;
	Result.Bottom:=Result.Bottom-BoxOffset;

end;


procedure TRectBox.SetInnerRect(Value:TRect);
var
	BoxOffset:Integer;
	Rect:TRect;
begin
	BoxOffset:=GetBoxOffset;

	Rect:=Value;
	Rect.Left:=Rect.Left-BoxOffset;
	Rect.Top:=Rect.Top-BoxOffset;
	Rect.Right:=Rect.Right+BoxOffset;
	Rect.Bottom:=Rect.Bottom+BoxOffset;

	BoundsRect:=Rect;
	OnRectChanged;
end;


procedure TRectBox.SetFrameType(Value: TFrameType);
begin
	if FFrameType = Value then exit;
	FFrameType := Value;
	Invalidate;
end;

procedure TRectBox.SetBoxSize(Value: Integer);
begin
	if Value = FBoxSize then exit;
	if value < 2 then exit;
	if value > 50 then exit;
	FBoxSize := ((Value div 2) * 2) + 1;
	invalidate;
end;

procedure TRectBox.SetBoxColor(Value: Tcolor);
begin
	if Value = FBoxColor then exit;
	FBoxColor := Value;
	invalidate;
end;

procedure TRectBox.SetActiveFrameColor(Value: Tcolor);
begin
	if Value = FActiveFrameColor then exit;
	FActiveFrameColor := Value;
	invalidate;
end;

procedure TRectBox.SetPassiveFrameColor(Value: Tcolor);
begin
	if Value = FPassiveFrameColor then exit;
	FPassiveFrameColor := Value;
	invalidate;
end;


procedure TRectBox.SetDrawFrame(Value: Boolean);
begin
	if Value = FDrawFrame then exit;
	FDrawFrame := Value;
	invalidate;
end;


procedure TRectBox.SetDrawBoxes(Value: Boolean);
begin
	if Value = FDrawBoxes then exit;
	FDrawBoxes := Value;
	invalidate;
end;


procedure TRectBox.SetHideFrame(Value: Boolean);
begin
	if Value = FHideFrame then exit;
  FHideFrame := Value;
  Invalidate;
end;

end.
