{
Copyright 2000 (c) Aurlien GATEAU aka Bug*Killer
aurelien.gateau@mail.dotcom.fr

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit cutit_gui;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls,CutFile, ComCtrls, ToolWin, ImgList, ActnList,
  Menus,DocView,Properties, Spin, About;

const
	FORM_CAPTION = 'Cut It ! [%s]';

type
TFormCutIt = class(TForm,IView)
	OpenDialog: TOpenDialog;
	ImageList: TImageList;
    ScrollBox: TScrollBox;
    ToolBarMain: TToolBar;
	ToolButtonOpen: TToolButton;
    ToolButtonCutIt: TToolButton;
    ImagePreview: TImage;
    ToolButton1: TToolButton;
    ToolButtonSave: TToolButton;
    ToolButtonNew: TToolButton;
    ToolButtonProp: TToolButton;
    ToolButtonSaveAs: TToolButton;
    SaveDialog: TSaveDialog;
    ActionList: TActionList;
    ActNew: TAction;
    ActOpen: TAction;
    ActSave: TAction;
    ActSaveAs: TAction;
    ActProperties: TAction;
    ActCutIt: TAction;
    MainMenu: TMainMenu;
    MenuFile: TMenuItem;
    MenuNew: TMenuItem;
    MenuOpen: TMenuItem;
    MenuSave: TMenuItem;
    MenuSaveAs: TMenuItem;
    MenuEdit: TMenuItem;
    MenuProperties: TMenuItem;
    MenuCutIt: TMenuItem;
    ToolBarCut: TToolBar;
    ToolButtonAddCut: TToolButton;
    ToolButtonRemoveCut: TToolButton;
	SpinEditHeight: TSpinEdit;
	SpinEditWidth: TSpinEdit;
    SpinEditTop: TSpinEdit;
	SpinEditLeft: TSpinEdit;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    EditFilename: TEdit;
    Help1: TMenuItem;
    MenuAbout: TMenuItem;
    ImageList16x16: TImageList;
    ActionListCut: TActionList;
    ActAddCut: TAction;
    ActRemoveCut: TAction;
    MenuCut: TMenuItem;
    Addcut1: TMenuItem;
    Removecut1: TMenuItem;

	procedure ActOpenExecute(Sender: TObject);
    procedure ActSaveAsExecute(Sender: TObject);
    procedure ActSaveExecute(Sender: TObject);
    procedure ActCutItExecute(Sender: TObject);
    procedure ActPropertiesExecute(Sender: TObject);
	procedure ActNewExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ActAddCutExecute(Sender: TObject);
	procedure ActRemoveCutExecute(Sender: TObject);

	procedure SetCutSelected(Value:boolean);
    procedure SpinEditHeightChange(Sender: TObject);
    procedure SpinEditLeftChange(Sender: TObject);
	procedure SpinEditWidthChange(Sender: TObject);
    procedure SpinEditTopChange(Sender: TObject);
    procedure EditFilenameChange(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
private
	FCutFile:TCutFile;
	FFileOpened,FFileModified,FFileNamed,FCutSelected:boolean;

	procedure OpenFile(FileName:string);
	procedure UpdateGUI();
	procedure OnDocMessage(Msg:TDVMsg);
	function ConfirmClose:boolean;
public
	constructor Create(AOwner: TComponent); override;
	destructor Destroy; override;

	property CutSelected: boolean read FCutSelected write SetCutSelected;
end;

var
   FormCutIt:TFormCutIt;

implementation

uses CutBox;

{$R *.DFM}

{

The form constructor.
If one param is passed in the cmd line,
it tries to load it as a CutIt file.

}
constructor TFormCutIt.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);

	FFileOpened:=False;
	FFileModified:=False;
	FFileNamed:=False;
	FCutSelected:=False;

// Set the double buffer for the scroll box
	ScrollBox.DoubleBuffered:=true;

// Load the param
	if ParamCount()=1 then
	begin
		OpenFile(ParamStr(1));
	end;

	UpdateGUI();
end;


{

The form destructor
Simply free the TCutFile object and calls the parent destructor

}
destructor TFormCutIt.Destroy;
begin
    FCutFile.free;
    inherited Destroy;
end;


procedure TFormCutIt.OpenFile(FileName:string);
var
	Num,NbCuts:Integer;
    CutBox:TCutBox;
begin
// Create the cut file object
	FCutFile.Free;
	FCutFile:=TCutFile.Create;

// Load it
	try
		FCutFile.Load(FileName);
	except
		on E:EFOpenError do
		begin
			FCutFile.Free();
			FCutFile:=nil;
			Application.MessageBox(PChar(E.Message),PChar(Application.Title),MB_ICONWARNING);
		end;
	end;

// Get the main bitmap
	ImagePreview.Picture.Bitmap:=FCutFile.Bitmap;

// Delete old cut boxes
	for Num:=ScrollBox.Controlcount-1 downto 0 do begin
		if ScrollBox.Controls[Num] is TCutBox then begin
			ScrollBox.Controls[Num].free;
		end;
	end;

// Create the new cut boxes
	NbCuts:=FCutFile.Count;
	for Num:=0 to NbCuts-1 do begin
		CutBox:=TCutBox.CreateFromCut(self,TCut(FCutFile[Num]),self);
		CutBox.Parent:=ScrollBox;
		TCut(FCutFile[Num]).Doc.AddView(CutBox);
	end;

// Button settings
	FFileOpened:=True;
	FFileModified:=False;
	FFileNamed:=True;
	FCutSelected:=False;
	UpdateGUI;

// Set our form as a view
	FCutFile.Doc.AddView(self);
end;


procedure TFormCutIt.OnDocMessage(Msg:TDVMsg);
begin
	FFileModified:=True;
	UpdateGUI;
end;


procedure TFormCutIt.UpdateGUI;
var
	TitleFileName:string;
	TitleString:string;
	CutRect:TRect;
begin
// Buttons
	ActSave.Enabled:=FFileOpened and FFileNamed and FFileModified;
	ActSaveAs.Enabled:=FFileOpened;
	ActProperties.Enabled:=FFileOpened;
	ActCutIt.Enabled:=FFileOpened;
	ActAddCut.Enabled:=FFileOpened;
	ActRemoveCut.Enabled:=FFileOpened and FCutSelected;

// Toolbar
	if FCutSelected then begin
		CutRect:=TCutBox(TCutBox.GetLastFocused).Cut.Rect;
		SpinEditLeft.Enabled:=true;
		SpinEditTop.Enabled:=true;
		SpinEditWidth.Enabled:=true;
		SpinEditHeight.Enabled:=true;
		EditFilename.Enabled:=true;

		SpinEditLeft.Value:=CutRect.Left;
		SpinEditTop.Value:=CutRect.Top;
		SpinEditWidth.Value:=CutRect.Right-CutRect.Left;
		SpinEditHeight.Value:=CutRect.Bottom-CutRect.Top;

		EditFilename.Text:=TCutBox(TCutBox.GetLastFocused).Cut.FileName;
	end else begin
		SpinEditLeft.Value:=0;
		SpinEditTop.Value:=0;
		SpinEditWidth.Value:=0;
		SpinEditHeight.Value:=0;
		EditFilename.Text:='';

		SpinEditLeft.Enabled:=false;
		SpinEditTop.Enabled:=false;
		SpinEditWidth.Enabled:=false;
		SpinEditHeight.Enabled:=false;
		EditFilename.Enabled:=false;
	end;


// Caption
	if FFileOpened then
	begin
		if FFileNamed then
			TitleFileName:= FCutFile.IniFileName
		else
			TitleFileName:='Unnamed';

		if FFileModified then
			TitleFileName:=TitleFileName + '*';
	end
	else
	begin
		TitleFileName:= 'No file opened';
	end;

	FmtStr(TitleString,FORM_CAPTION,[TitleFileName]);
	Caption:=TitleString;
end;


procedure TFormCutIt.ActOpenExecute(Sender: TObject);
var
	FileName:String;
begin
	if not ConfirmClose then exit;

// Select file
	if OpenDialog.Execute then
		FileName:=OpenDialog.FileName
	else
		exit;

// Open it
	OpenFile(FileName);
	UpdateGUI();
end;

procedure TFormCutIt.ActSaveAsExecute(Sender: TObject);
var
	FileName:string;
begin
// Select file
	if SaveDialog.Execute then
		FileName:=SaveDialog.FileName
	else
		exit;

// Save
	FCutFile.SaveAs(FileName);
	FFileNamed:=true;
	FFileModified:=false;
	UpdateGUI();

end;

procedure TFormCutIt.ActSaveExecute(Sender: TObject);
begin
	FCutFile.Save;
	FFileModified:=false;
	UpdateGUI();
end;

procedure TFormCutIt.ActCutItExecute(Sender: TObject);
begin
	Cursor:=crHourGlass;
	FCutFile.DoCut;
	Cursor:=crDefault;
end;

procedure TFormCutIt.ActPropertiesExecute(Sender: TObject);
var
	Prop:TFormProperties;
	Value:Integer;
begin
	Prop:=TFormProperties.CreateFromCutFile(self,FCutFile);
	Value:=Prop.ShowModal;
	Prop.free;
	
	if Value=mrOk then
	begin
		FFileModified:=true;
		ImagePreview.Picture.Bitmap:=FCutFile.Bitmap;
		UpdateGUI;
	end;

end;


function TFormCutIt.ConfirmClose:boolean;
begin
// Warn if not saved
	if FFileOpened and FFileModified then
		Result:=MessageDlg('The current file has not been saved, are you sure you want to continue ?',mtConfirmation,[mbNo,mbYes], 0)=mrYes
	else
		Result:=true;
end;


procedure TFormCutIt.ActNewExecute(Sender: TObject);
var
	Prop:TFormProperties;
	Value:Integer;
	CutFile:TCutFile;
begin
	if not ConfirmClose then exit;

// Create a new cut file
	CutFile:=TCutFile.Create;

// Define the main properties
	Prop:=TFormProperties.CreateFromCutFile(self,CutFile);
	Value:=Prop.ShowModal;
	Prop.free;

	if Value=mrOk then
	begin
	// Replace FCutFile with the new one
		FCutFile.Free;
		FCutFile:=CutFile;

	// Set our form as a view
		FCutFile.Doc.AddView(self);

	// Update vars
		FFileNamed:=false;
		FFileOpened:=true;
		FFileModified:=true;
		ImagePreview.Picture.Bitmap:=FCutFile.Bitmap;
		UpdateGUI;
	end
	else
	begin
	// Delete the new cutfile
		CutFile.Free;
	end;
end;


procedure TFormCutIt.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
	CanClose:=ConfirmClose;
end;


procedure TFormCutIt.ActAddCutExecute(Sender: TObject);
var
	Cut:TCut;
	CutBox:TCutBox;
begin
// Create a cut
	Cut:=FCutFile.AddCut;

// Create a cut box for it
	CutBox:=TCutBox.CreateFromCut(self,Cut,self);
	CutBox.Parent:=ScrollBox;
	Cut.Doc.AddView(CutBox);

// Update the GUI
	FFileModified:=true;
	UpdateGUI;
end;


procedure TFormCutIt.ActRemoveCutExecute(Sender: TObject);
var
	CutBox:TCutBox;
	Cut:TCut;
begin
// Get the necessary info
	CutBox:=TCutBox(TCutBox.GetLastFocused);
	Cut:=CutBox.Cut;


// Update the GUI
	FFileModified:=true;
	FCutSelected:=false;
	UpdateGUI;

// Remove the cut box
	CutBox.Free;

// Remove the cut
	FCutFile.RemoveCut(Cut);
end;


procedure TFormCutIt.SetCutSelected(Value:boolean);
begin
	FCutSelected:=true;
	UpdateGUI;
end;


procedure TFormCutIt.SpinEditLeftChange(Sender: TObject);
var
	Rect:TRect;
	Width:integer;
begin
	if TCutBox.GetLastFocused=nil then exit;

	Rect:=TCutBox(TCutBox.GetLastFocused).Cut.Rect;
	if Rect.Left=SpinEditLeft.value then exit;

	Width:=Rect.Right-Rect.Left;

	Rect.left:=SpinEditLeft.value;
	Rect.Right:=Rect.Left+Width;

	TCutBox(TCutBox.GetLastFocused).Cut.Rect:=Rect;
end;

procedure TFormCutIt.SpinEditTopChange(Sender: TObject);
var
	Rect:TRect;
	Height:integer;
begin
	if TCutBox.GetLastFocused=nil then exit;

	Rect:=TCutBox(TCutBox.GetLastFocused).Cut.Rect;
	if Rect.Top=SpinEditTop.value then exit;

	Height:=Rect.Bottom-Rect.Top;

	Rect.top:=SpinEditTop.value;
	Rect.Bottom:=Rect.top+Height;

	TCutBox(TCutBox.GetLastFocused).Cut.Rect:=Rect;
end;

procedure TFormCutIt.SpinEditWidthChange(Sender: TObject);
var
	Rect:TRect;
begin
	if TCutBox.GetLastFocused=nil then exit;

	Rect:=TCutBox(TCutBox.GetLastFocused).Cut.Rect;
	if Rect.Right-Rect.Left=SpinEditWidth.value then exit;

	Rect.right:=SpinEditWidth.value + Rect.left;
	TCutBox(TCutBox.GetLastFocused).Cut.Rect:=Rect;
end;

procedure TFormCutIt.SpinEditHeightChange(Sender: TObject);
var
	Rect:TRect;
begin
	if TCutBox.GetLastFocused=nil then exit;

	Rect:=TCutBox(TCutBox.GetLastFocused).Cut.Rect;
	if Rect.Bottom-Rect.Top=SpinEditHeight.value then exit;

	Rect.Bottom:=SpinEditHeight.value + Rect.top;
	TCutBox(TCutBox.GetLastFocused).Cut.Rect:=Rect;
end;

procedure TFormCutIt.EditFilenameChange(Sender: TObject);
begin
	if TCutBox.GetLastFocused=nil then exit;
	if TCutBox(TCutBox.GetLastFocused).Cut.FileName=EditFilename.text then exit;
	
	TCutBox(TCutBox.GetLastFocused).Cut.FileName:=EditFilename.text;
end;

procedure TFormCutIt.MenuAboutClick(Sender: TObject);
var
	FormAbout: TFormAbout;
begin
	FormAbout:=TFormAbout.Create(self);
	FormAbout.ShowModal;
	FormAbout.free;
end;

end.
